@@ -184,7 +184,7 @@ namespace Banshee.MediaEngine
 
         private void OnMetadataServiceHaveResult (object o, MetadataLookupResultArgs args)
         {
-            if (CurrentTrack != null && args.Track == CurrentTrack) {
+            if (CurrentTrack != null && CurrentTrack.TrackEqual (args.Track as TrackInfo)) {
                 foreach (StreamTag tag in args.ResultTags) {
                     StreamTagger.TrackInfoMerge (CurrentTrack, tag);
                 }
@@ -567,6 +567,16 @@ namespace Banshee.MediaEngine
             set { active_engine.Position = value; }
         }
 
+        public byte Rating {
+            get { return (byte)(CurrentTrack == null ? 0 : CurrentTrack.Rating); }
+            set {
+                if (CurrentTrack != null) {
+                    CurrentTrack.Rating = (int)Math.Min (5u, value);
+                    CurrentTrack.Save ();
+                }
+            }
+        }
+
         public bool CanSeek {
             get { return active_engine.CanSeek; }
         }
