@@ -6,24 +6,24 @@
  *  Gabriel Burt (gburt@novell.com)
  ****************************************************************************/
 
-/*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW: 
+/*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW:
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
- *  copy of this software and associated documentation files (the "Software"),  
- *  to deal in the Software without restriction, including without limitation  
- *  the rights to use, copy, modify, merge, publish, distribute, sublicense,  
- *  and/or sell copies of the Software, and to permit persons to whom the  
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
  *  Software is furnished to do so, subject to the following conditions:
  *
- *  The above copyright notice and this permission notice shall be included in 
+ *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
- *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
- *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
- *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
- *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  *  DEALINGS IN THE SOFTWARE.
  */
 
@@ -33,9 +33,9 @@ using System.Runtime.InteropServices;
 
 namespace Mtp
 {
-	public class Album : AbstractTrackList
-	{
-		private AlbumStruct album;
+    public class Album : AbstractTrackList
+    {
+        private AlbumStruct album;
 
         public uint AlbumId {
             get { return Saved ? album.album_id : 0; }
@@ -43,30 +43,37 @@ namespace Mtp
 
         public override string Name {
             get { return album.name; }
-            set {
-                album.name = value;
-            }
+            set { album.name = value; }
         }
 
         public string Artist {
             get { return album.artist; }
-            set {
-                album.artist = value;
-            }
+            set { album.artist = value; }
         }
 
         public string Genre {
             get { return album.genre; }
+            set { album.genre = value; }
+        }
+
+        public string Composer {
+            get {
+#if LIBMTP8
+                return album.composer;
+#else
+                return null;
+#endif
+            }
             set {
-                album.genre = value;
+#if LIBMTP8
+                album.composer = value;
+#endif
             }
         }
 
         public override uint Count {
             get { return album.no_tracks; }
-            protected set {
-                album.no_tracks = value;
-            }
+            protected set { album.no_tracks = value; }
         }
 
         protected override IntPtr TracksPtr {
@@ -74,14 +81,14 @@ namespace Mtp
             set { album.tracks = value; }
         }
 
-        public Album (MtpDevice device, string name, string artist, string genre) : base (device, name)
+        public Album (MtpDevice device, string name, string artist, string genre, string composer) : base (device, name)
         {
-            this.album = new AlbumStruct ();
-            TracksPtr = IntPtr.Zero;
             Name = name;
             Artist = artist;
             Genre = genre;
+            Composer = composer;
             Count = 0;
+            TracksPtr = IntPtr.Zero;
         }
 
         internal Album (MtpDevice device, AlbumStruct album) : base (device, album.tracks, album.no_tracks)
@@ -101,7 +108,7 @@ namespace Mtp
                 if (cover_art == null) {
                     return;
                 }
-                
+
                 FileSampleData cover = new FileSampleData ();
                 cover.data = Marshal.AllocHGlobal (Marshal.SizeOf (typeof (byte)) * cover_art.Length);
                 Marshal.Copy (cover_art, 0, cover.data, cover_art.Length);
@@ -111,7 +118,7 @@ namespace Mtp
                 cover.filetype = FileType.JPEG;
 
                 if (FileSample.LIBMTP_Send_Representative_Sample (Device.Handle, AlbumId, ref cover) != 0) {
-                    //Console.WriteLine ("failed to send representative sample file");
+                    //Console.WriteLine ("Failed to send representative sample file for album {0} (id {1})", Name, AlbumId);
                 }
                 Marshal.FreeHGlobal (cover.data);
             }
@@ -130,10 +137,10 @@ namespace Mtp
         {
             return LIBMTP_Update_Album (Device.Handle, ref album);
         }
-        
+
         public void Remove ()
         {
-			MtpDevice.LIBMTP_Delete_Object(Device.Handle, AlbumId);
+            MtpDevice.LIBMTP_Delete_Object(Device.Handle, AlbumId);
         }
 
         public override string ToString ()
@@ -151,29 +158,29 @@ namespace Mtp
             }
         }
 
-		[DllImport("libmtp.dll")]
-		internal static extern IntPtr LIBMTP_new_album_t (); // LIBMTP_album_t*
+        [DllImport("libmtp.dll")]
+        internal static extern IntPtr LIBMTP_new_album_t (); // LIBMTP_album_t*
 
-		[DllImport("libmtp.dll")]
-		internal static extern void LIBMTP_destroy_album_t (ref AlbumStruct album);
+        [DllImport("libmtp.dll")]
+        internal static extern void LIBMTP_destroy_album_t (ref AlbumStruct album);
 
-		[DllImport("libmtp.dll")]
-		internal static extern IntPtr LIBMTP_Get_Album_List (MtpDeviceHandle handle); // LIBMTP_album_t*
+        [DllImport("libmtp.dll")]
+        internal static extern IntPtr LIBMTP_Get_Album_List (MtpDeviceHandle handle); // LIBMTP_album_t*
 
-		[DllImport("libmtp.dll")]
-		internal static extern IntPtr LIBMTP_Get_Album (MtpDeviceHandle handle, uint albumId); // LIBMTP_album_t*
+        [DllImport("libmtp.dll")]
+        internal static extern IntPtr LIBMTP_Get_Album (MtpDeviceHandle handle, uint albumId); // LIBMTP_album_t*
 
 #if LIBMTP8
-		[DllImport("libmtp.dll")]
-		internal static extern int LIBMTP_Create_New_Album (MtpDeviceHandle handle, ref AlbumStruct album);
+        [DllImport("libmtp.dll")]
+        internal static extern int LIBMTP_Create_New_Album (MtpDeviceHandle handle, ref AlbumStruct album);
 #else
-		[DllImport("libmtp.dll")]
-		internal static extern int LIBMTP_Create_New_Album (MtpDeviceHandle handle, ref AlbumStruct album, uint parentId);
+        [DllImport("libmtp.dll")]
+        internal static extern int LIBMTP_Create_New_Album (MtpDeviceHandle handle, ref AlbumStruct album, uint parentId);
 #endif
 
-		[DllImport("libmtp.dll")]
-		internal static extern int LIBMTP_Update_Album (MtpDeviceHandle handle, ref AlbumStruct album);
-	}
+        [DllImport("libmtp.dll")]
+        internal static extern int LIBMTP_Update_Album (MtpDeviceHandle handle, ref AlbumStruct album);
+    }
 
     [StructLayout(LayoutKind.Sequential)]
     internal struct AlbumStruct
@@ -190,12 +197,17 @@ namespace Mtp
         [MarshalAs(UnmanagedType.LPStr)]
         public string artist;
 
+#if LIBMTP8
+        [MarshalAs(UnmanagedType.LPStr)]
+        public string composer;
+#endif
+
         [MarshalAs(UnmanagedType.LPStr)]
         public string genre;
 
         public IntPtr tracks;
         public uint no_tracks;
 
-        public IntPtr next; // LIBMTP_album_t*
+        public IntPtr next;
     }
 }
